@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,11 +35,13 @@
 #include "UsrUtils.h"
 #include "StrUtils.h"
 #include "POP3Utils.h"
-#include "Queue.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
 #include "UsrMailList.h"
 #include "POP3GwLink.h"
 #include "MailDomains.h"
-#include "SMAILUtils.h"
+#include "AliasDomain.h"
 #include "SMTPUtils.h"
 #include "MailConfig.h"
 #include "AppDefines.h"
@@ -65,6 +67,7 @@
 #define CTRL_QUIT_CMD_EXIT      1
 #define CTRL_LISTFOLLOW_RESULT  100
 #define CTRL_WAITDATA_RESULT    101
+#define CTRL_VAR_DROP_VALUE     ".|rm"
 #define CTRL_SERVER_NAME        "[" APP_NAME_VERSION_OS_STR " CTRL Server]"
 
 
@@ -102,7 +105,7 @@ static int      CTRLVSendCmdResult(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock, i
                         char const * pszFormat,...);
 static int      CTRLSendCmdResult(BSOCK_HANDLE hBSock, int iErrorCode, char const * pszMessage,
                         int iTimeout);
-static char    *CTRLGetAccountsFilePath(char *pszAccFilePath);
+static char    *CTRLGetAccountsFilePath(char *pszAccFilePath, int iMaxPath);
 static int      CTRLAccountCheck(CTRLConfig * pCTRLCfg, char const * pszUsername,
                         char const * pszPassword, char const * pszTimeStamp);
 static int      CTRLLogin(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
@@ -135,6 +138,8 @@ static int      CTRLDo_usersetmproc(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_userauth(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_userstat(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_mluseradd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_mluserdel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
@@ -165,12 +170,32 @@ static int      CTRLDo_poplnklist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_poplnkenable(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
+static int      CTRLCheckRelativePath(char const * pszPath);
+static int      CTRLDo_filelist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_cfgfileget(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_cfgfileset(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
 static int      CTRLDo_frozlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_frozsubmit(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_frozdel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_frozgetlog(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_frozgetmsg(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_etrn(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_aliasdomainadd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_aliasdomaindel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+static int      CTRLDo_aliasdomainlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount);
+
 
 
 
@@ -233,8 +258,8 @@ static int      CTRLCheckPeerIP(SYS_SOCKET SockFD)
 
     char            szIPMapFile[SYS_MAX_PATH] = "";
 
-    CfgGetRootPath(szIPMapFile);
-    strcat(szIPMapFile, CTRL_IPMAP_FILE);
+    CfgGetRootPath(szIPMapFile, sizeof(szIPMapFile));
+    StrSNCat(szIPMapFile, CTRL_IPMAP_FILE);
 
     if (SysExistFile(szIPMapFile))
     {
@@ -269,12 +294,14 @@ static int      CTRLLogSession(char const * pszUsername, char const * pszPasswor
         return (ErrGetErrorCode());
 
 
+    char            szIP[128] = "???.???.???.???";
+
     MscFileLog(CTRL_LOG_FILE, "\"%s\""
             "\t\"%s\""
             "\t\"%s\""
             "\t\"%s\""
             "\t\"%s\""
-            "\n", SysInetNToA(PeerInfo), pszUsername, pszPassword, szTime,
+            "\n", SysInetNToA(PeerInfo, szIP), pszUsername, pszPassword, szTime,
             (iStatus == 0) ? "REQ" : ((iStatus > 0) ? "AUTH" : "FAIL"));
 
 
@@ -473,8 +500,11 @@ static unsigned int CTRLClientThread(void *pThreadData)
         return (ErrorPop());
     }
 
+
+    char            szIP[128] = "???.???.???.???";
+
     SysLogMessage(LOG_LEV_MESSAGE, "CTRL client connection from [%s]\n",
-            SysInetNToA(PeerInfo));
+            SysInetNToA(PeerInfo, szIP));
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -484,7 +514,7 @@ static unsigned int CTRLClientThread(void *pThreadData)
 
 
     SysLogMessage(LOG_LEV_MESSAGE, "CTRL client exit [%s]\n",
-            SysInetNToA(PeerInfo));
+            SysInetNToA(PeerInfo, szIP));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Unlink socket from the bufferer and close it
@@ -567,12 +597,12 @@ static int      CTRLSendCmdResult(BSOCK_HANDLE hBSock, int iErrorCode, char cons
 
 
 
-static char    *CTRLGetAccountsFilePath(char *pszAccFilePath)
+static char    *CTRLGetAccountsFilePath(char *pszAccFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszAccFilePath);
+    CfgGetRootPath(pszAccFilePath, iMaxPath);
 
-    strcat(pszAccFilePath, CTRL_ACCOUNTS_FILE);
+    StrNCat(pszAccFilePath, CTRL_ACCOUNTS_FILE, iMaxPath);
 
     return (pszAccFilePath);
 
@@ -587,11 +617,12 @@ static int      CTRLAccountCheck(CTRLConfig * pCTRLCfg, char const * pszUsername
 
     char            szAccFilePath[SYS_MAX_PATH] = "";
 
-    CTRLGetAccountsFilePath(szAccFilePath);
+    CTRLGetAccountsFilePath(szAccFilePath, sizeof(szAccFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAccFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAccFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -762,8 +793,10 @@ static int      CTRLHandleSession(SHB_HANDLE hShbCTRL, BSOCK_HANDLE hBSock,
 
     SysGetSockInfo(BSckGetAttachedSocket(hBSock), SockInfo);
 
+    char            szIP[128] = "???.???.???.???";
+
     sprintf(szTimeStamp, "<%lu.%lu@%s>",
-            (unsigned long) time(NULL), SysGetCurrentThreadId(), SysInetNToA(SockInfo));
+            (unsigned long) time(NULL), SysGetCurrentThreadId(), SysInetNToA(SockInfo, szIP));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Welcome
@@ -871,6 +904,8 @@ static int      CTRLProcessCommand(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         iCmdResult = CTRLDo_usersetmproc(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "userauth") == 0)
         iCmdResult = CTRLDo_userauth(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "userstat") == 0)
+        iCmdResult = CTRLDo_userstat(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "aliasadd") == 0)
         iCmdResult = CTRLDo_aliasadd(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "aliasdel") == 0)
@@ -903,12 +938,30 @@ static int      CTRLProcessCommand(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         iCmdResult = CTRLDo_poplnklist(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "poplnkenable") == 0)
         iCmdResult = CTRLDo_poplnkenable(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "filelist") == 0)
+        iCmdResult = CTRLDo_filelist(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "cfgfileget") == 0)
         iCmdResult = CTRLDo_cfgfileget(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "cfgfileset") == 0)
         iCmdResult = CTRLDo_cfgfileset(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "frozlist") == 0)
         iCmdResult = CTRLDo_frozlist(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "frozsubmit") == 0)
+        iCmdResult = CTRLDo_frozsubmit(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "frozdel") == 0)
+        iCmdResult = CTRLDo_frozdel(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "frozgetlog") == 0)
+        iCmdResult = CTRLDo_frozgetlog(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "frozgetmsg") == 0)
+        iCmdResult = CTRLDo_frozgetmsg(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "aliasdomainadd") == 0)
+        iCmdResult = CTRLDo_aliasdomainadd(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "aliasdomaindel") == 0)
+        iCmdResult = CTRLDo_aliasdomaindel(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "aliasdomainlist") == 0)
+        iCmdResult = CTRLDo_aliasdomainlist(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
+    else if (stricmp(ppszTokens[0], "etrn") == 0)
+        iCmdResult = CTRLDo_etrn(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "noop") == 0)
         iCmdResult = CTRLDo_noop(pCTRLCfg, hBSock, ppszTokens, iTokensCount);
     else if (stricmp(ppszTokens[0], "quit") == 0)
@@ -1088,13 +1141,6 @@ static int      CTRLDo_aliasadd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         return (ERR_BAD_CTRL_COMMAND);
     }
 
-    if (MDomIsHandledDomain(ppszTokens[1]) < 0)
-    {
-        ErrorPush();
-        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
-        return (ErrorPop());
-    }
-
 ///////////////////////////////////////////////////////////////////////////////
 //  Check real user account existence
 ///////////////////////////////////////////////////////////////////////////////
@@ -1118,6 +1164,19 @@ static int      CTRLDo_aliasadd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
     UsrFreeUserInfo(pUI);
 
+///////////////////////////////////////////////////////////////////////////////
+//  Check if we're overlapping an existing users with the new alias
+///////////////////////////////////////////////////////////////////////////////
+    if ((pUI = UsrGetUserByName(ppszTokens[1], ppszTokens[2])) != NULL)
+    {
+        UsrFreeUserInfo(pUI);
+
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_USER_EXIST);
+
+        ErrSetErrorCode(ERR_USER_EXIST);
+        return (ERR_USER_EXIST);
+    }
+
 
     AliasInfo      *pAI = UsrAllocAlias(ppszTokens[1], ppszTokens[2], ppszTokens[3]);
 
@@ -1160,13 +1219,6 @@ static int      CTRLDo_aliasdel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         return (ERR_BAD_CTRL_COMMAND);
     }
 
-    if (MDomIsHandledDomain(ppszTokens[1]) < 0)
-    {
-        ErrorPush();
-        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
-        return (ErrorPop());
-    }
-
 
     if (UsrRemoveAlias(ppszTokens[1], ppszTokens[2]) < 0)
     {
@@ -1366,12 +1418,23 @@ static int      CTRLDo_uservarsset(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
     for (int ii = 3; ii < (iTokensCount - 1); ii += 2)
     {
-        if (UsrSetUserInfoVar(pUI, ppszTokens[ii], ppszTokens[ii + 1]) < 0)
+///////////////////////////////////////////////////////////////////////////////
+//  Check if the variable deletion is requested
+///////////////////////////////////////////////////////////////////////////////
+        if (strcmp(ppszTokens[ii + 1], CTRL_VAR_DROP_VALUE) == 0)
+            UsrDelUserInfoVar(pUI, ppszTokens[ii]);
+        else
         {
-            ErrorPush();
-            UsrFreeUserInfo(pUI);
-            CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
-            return (ErrorPop());
+///////////////////////////////////////////////////////////////////////////////
+//  Set user variable
+///////////////////////////////////////////////////////////////////////////////
+            if (UsrSetUserInfoVar(pUI, ppszTokens[ii], ppszTokens[ii + 1]) < 0)
+            {
+                ErrorPush();
+                UsrFreeUserInfo(pUI);
+                CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+                return (ErrorPop());
+            }
         }
     }
 
@@ -1659,6 +1722,79 @@ static int      CTRLDo_userauth(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
 
 
+static int      CTRLDo_userstat(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount != 3)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check real user account existence
+///////////////////////////////////////////////////////////////////////////////
+    char            szRealAddress[MAX_ADDR_NAME] = "";
+    UserInfo       *pUI = UsrGetUserByNameOrAlias(ppszTokens[1], ppszTokens[2],
+                            szRealAddress);
+
+    if (pUI == NULL)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Get mailbox infos
+///////////////////////////////////////////////////////////////////////////////
+    unsigned long   ulMBSize = 0,
+                    ulNumMessages = 0;
+
+    if (UPopGetMailboxSize(pUI, ulMBSize, ulNumMessages) < 0)
+    {
+        ErrorPush();
+        UsrFreeUserInfo(pUI);
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+    SYS_INET_ADDR   LastLoginAddr;
+    char            szIPAddr[128] = "0.0.0.0";
+
+    if (UPopGetLastLoginAddress(pUI, &LastLoginAddr) == 0)
+        SysInetNToA(LastLoginAddr, szIPAddr);
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_LISTFOLLOW_RESULT);
+
+
+    if ((BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"RealAddress\"\t\"%s\"",
+            szRealAddress) < 0) ||
+            (BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"MailboxSize\"\t\"%lu\"",
+            ulMBSize) < 0) ||
+            (BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"MailboxMessages\"\t\"%lu\"",
+            ulNumMessages) < 0) ||
+            (BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"LastLoginIP\"\t\"%s\"",
+            szIPAddr) < 0))
+    {
+        ErrorPush();
+        UsrFreeUserInfo(pUI);
+        return (ErrorPop());
+    }
+
+    UsrFreeUserInfo(pUI);
+
+    BSckSendString(hBSock, ".", pCTRLCfg->iTimeout);
+
+    return (0);
+
+}
+
+
+
 static int      CTRLDo_mluseradd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount)
 {
@@ -1885,7 +2021,7 @@ static int      CTRLDo_domainadd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     }
 
 
-    char            szDomain[256] = "";
+    char            szDomain[MAX_HOST_NAME] = "";
 
     StrSNCpy(szDomain, ppszTokens[1]);
     StrLower(szDomain);
@@ -1919,7 +2055,7 @@ static int      CTRLDo_domaindel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     }
 
 
-    char            szDomain[256] = "";
+    char            szDomain[MAX_HOST_NAME] = "";
 
     StrSNCpy(szDomain, ppszTokens[1]);
     StrLower(szDomain);
@@ -1945,14 +2081,6 @@ static int      CTRLDo_domainlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount)
 {
 
-    if (iTokensCount != 1)
-    {
-        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
-        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
-        return (ERR_BAD_CTRL_COMMAND);
-    }
-
-
     DOMLS_HANDLE    hDomainsDB = MDomOpenDB();
 
     if (hDomainsDB == INVALID_DOMLS_HANDLE)
@@ -1972,17 +2100,19 @@ static int      CTRLDo_domainlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     {
         do
         {
-            char            szDomainLine[512] = "";
-
-            sprintf(szDomainLine, "\"%s\"", pszDomain);
-
-            if (BSckSendString(hBSock, szDomainLine, pCTRLCfg->iTimeout) < 0)
+            if ((iTokensCount < 2) || StrStringsRIWMatch(&ppszTokens[1], pszDomain))
             {
-                ErrorPush();
-                MDomCloseDB(hDomainsDB);
-                return (ErrorPop());
-            }
+                char            szDomainLine[512] = "";
 
+                sprintf(szDomainLine, "\"%s\"", pszDomain);
+
+                if (BSckSendString(hBSock, szDomainLine, pCTRLCfg->iTimeout) < 0)
+                {
+                    ErrorPush();
+                    MDomCloseDB(hDomainsDB);
+                    return (ErrorPop());
+                }
+            }
         } while ((pszDomain = MDomGetNextDomain(hDomainsDB)) != NULL);
     }
 
@@ -2126,7 +2256,7 @@ static int      CTRLDo_custdomlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
     char            szCustomPath[SYS_MAX_PATH] = "";
 
-    USmlGetDomainCustomDir(szCustomPath, 0);
+    USmlGetDomainCustomDir(szCustomPath, sizeof(szCustomPath), 0);
 
 
     char            szCustFileName[SYS_MAX_PATH] = "";
@@ -2364,7 +2494,7 @@ static int      CTRLDo_poplnklist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
             if (((pszDomain == NULL) || (stricmp(pPopLnk->pszDomain, pszDomain) == 0)) &&
                     ((pszName == NULL) || (stricmp(pPopLnk->pszName, pszName) == 0)))
             {
-                char const     *pszEnable = (GwLkCheckEnabled(pPopLnk) == 0) ? "ON": "OFF";
+                char const     *pszEnable = (GwLkCheckEnabled(pPopLnk) == 0) ? "ON" : "OFF";
                 char            szLinkLine[2048] = "";
 
                 sprintf(szLinkLine,
@@ -2436,6 +2566,105 @@ static int      CTRLDo_poplnkenable(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
 
 
+static int      CTRLCheckRelativePath(char const * pszPath)
+{
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check 101 tricky path
+///////////////////////////////////////////////////////////////////////////////
+    if (strstr(pszPath, "..") != NULL)
+    {
+        ErrSetErrorCode(ERR_BAD_RELATIVE_PATH);
+        return (ERR_BAD_RELATIVE_PATH);
+    }
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_filelist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount != 3)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check relative path syntax
+///////////////////////////////////////////////////////////////////////////////
+    if (CTRLCheckRelativePath(ppszTokens[1]) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
+        return (ErrorPop());
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Setup listing file path
+///////////////////////////////////////////////////////////////////////////////
+    char            szRelativePath[SYS_MAX_PATH] = "",
+                    szFullPath[SYS_MAX_PATH] = "";
+
+    StrSNCpy(szRelativePath, ppszTokens[1]);
+    MscTranslatePath(szRelativePath);
+
+    CfgGetFullPath(szRelativePath, szFullPath, sizeof(szFullPath));
+    DelFinalSlash(szFullPath);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Check directory existance
+///////////////////////////////////////////////////////////////////////////////
+    if (!SysExistDir(szFullPath))
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_LISTDIR_NOT_FOUND);
+        ErrSetErrorCode(ERR_LISTDIR_NOT_FOUND);
+        return (ERR_LISTDIR_NOT_FOUND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Send command continue response
+///////////////////////////////////////////////////////////////////////////////
+    CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_LISTFOLLOW_RESULT);
+
+///////////////////////////////////////////////////////////////////////////////
+//  List files
+///////////////////////////////////////////////////////////////////////////////
+    char            szFileName[SYS_MAX_PATH] = "";
+    FSCAN_HANDLE    hFileScan = MscFirstFile(szFullPath, 0, szFileName);
+
+    if (hFileScan != INVALID_FSCAN_HANDLE)
+    {
+        do
+        {
+            if (!SYS_IS_VALID_FILENAME(szFileName) || !StrWildMatch(szFileName, ppszTokens[2]))
+                continue;
+
+            if (BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"%s\"", szFileName) < 0)
+            {
+                ErrorPush();
+                MscCloseFindFile(hFileScan);
+                return (ErrorPop());
+            }
+
+        } while (MscNextFile(hFileScan, szFileName));
+
+        MscCloseFindFile(hFileScan);
+    }
+
+    BSckSendString(hBSock, ".", pCTRLCfg->iTimeout);
+
+    return (0);
+
+}
+
+
+
 static int      CTRLDo_cfgfileget(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
                         char const * const * ppszTokens, int iTokensCount)
 {
@@ -2447,6 +2676,16 @@ static int      CTRLDo_cfgfileget(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         return (ERR_BAD_CTRL_COMMAND);
     }
 
+///////////////////////////////////////////////////////////////////////////////
+//  Check relative path syntax
+///////////////////////////////////////////////////////////////////////////////
+    if (CTRLCheckRelativePath(ppszTokens[1]) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
+        return (ErrorPop());
+    }
+
 ///////////////////////////////////////////////////////////////////////////////
 //  Setup client target file path
 ///////////////////////////////////////////////////////////////////////////////
@@ -2456,13 +2695,15 @@ static int      CTRLDo_cfgfileget(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     StrSNCpy(szRelativePath, ppszTokens[1]);
     MscTranslatePath(szRelativePath);
 
-    CfgGetFullPath(szRelativePath, szFullPath);
+    CfgGetFullPath(szRelativePath, szFullPath, sizeof(szFullPath));
+    DelFinalSlash(szFullPath);
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Share lock client target file
 ///////////////////////////////////////////////////////////////////////////////
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szFullPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szFullPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -2522,6 +2763,16 @@ static int      CTRLDo_cfgfileset(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
         return (ERR_BAD_CTRL_COMMAND);
     }
 
+///////////////////////////////////////////////////////////////////////////////
+//  Check relative path syntax
+///////////////////////////////////////////////////////////////////////////////
+    if (CTRLCheckRelativePath(ppszTokens[1]) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
+        return (ErrorPop());
+    }
+
 ///////////////////////////////////////////////////////////////////////////////
 //  Setup client target file path
 ///////////////////////////////////////////////////////////////////////////////
@@ -2531,7 +2782,8 @@ static int      CTRLDo_cfgfileset(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     StrSNCpy(szRelativePath, ppszTokens[1]);
     MscTranslatePath(szRelativePath);
 
-    CfgGetFullPath(szRelativePath, szFullPath);
+    CfgGetFullPath(szRelativePath, szFullPath, sizeof(szFullPath));
+    DelFinalSlash(szFullPath);
 
 
     CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_WAITDATA_RESULT);
@@ -2568,7 +2820,8 @@ static int      CTRLDo_cfgfileset(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 //  Exclusive lock client target file
 ///////////////////////////////////////////////////////////////////////////////
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szFullPath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szFullPath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -2624,7 +2877,7 @@ static int      CTRLDo_frozlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
 
     SysGetTmpFile(szListFile);
 
-    if (QueGetFrozenList(NULL, szListFile) < 0)
+    if (QueUtGetFrozenList(hSpoolQueue, szListFile) < 0)
     {
         ErrorPush();
         CheckRemoveFile(szListFile);
@@ -2649,3 +2902,358 @@ static int      CTRLDo_frozlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
     return (0);
 
 }
+
+
+
+static int      CTRLDo_frozsubmit(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount < 4)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Try to defroze frozen message
+///////////////////////////////////////////////////////////////////////////////
+    int             iLevel1 = atoi(ppszTokens[1]),
+                    iLevel2 = atoi(ppszTokens[2]);
+    char            szMessageFile[SYS_MAX_PATH] = "";
+
+    StrSNCpy(szMessageFile, ppszTokens[3]);
+
+    if (QueUtUnFreezeMessage(hSpoolQueue, iLevel1, iLevel2, szMessageFile) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, 0);
+
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_frozdel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount < 4)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Try to delete frozen message
+///////////////////////////////////////////////////////////////////////////////
+    int             iLevel1 = atoi(ppszTokens[1]),
+                    iLevel2 = atoi(ppszTokens[2]);
+    char            szMessageFile[SYS_MAX_PATH] = "";
+
+    StrSNCpy(szMessageFile, ppszTokens[3]);
+
+    if (QueUtDeleteFrozenMessage(hSpoolQueue, iLevel1, iLevel2, szMessageFile) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, 0);
+
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_frozgetlog(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount < 4)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Try to delete frozen message
+///////////////////////////////////////////////////////////////////////////////
+    int             iLevel1 = atoi(ppszTokens[1]),
+                    iLevel2 = atoi(ppszTokens[2]);
+    char            szMessageFile[SYS_MAX_PATH] = "";
+
+    StrSNCpy(szMessageFile, ppszTokens[3]);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Get log file snapshot
+///////////////////////////////////////////////////////////////////////////////
+    char            szFileSS[SYS_MAX_PATH] = "";
+
+    SysGetTmpFile(szFileSS);
+
+    if (QueUtGetFrozenLogFile(hSpoolQueue, iLevel1, iLevel2, szMessageFile, szFileSS) < 0)
+    {
+        ErrorPush();
+        CheckRemoveFile(szFileSS);
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_LISTFOLLOW_RESULT);
+
+
+///////////////////////////////////////////////////////////////////////////////
+//  Send client target file
+///////////////////////////////////////////////////////////////////////////////
+    if (MscSendTextFile(szFileSS, hBSock, pCTRLCfg->iTimeout) < 0)
+    {
+        ErrorPush();
+        SysRemove(szFileSS);
+        return (ErrorPop());
+    }
+
+    SysRemove(szFileSS);
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_frozgetmsg(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount < 4)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Try to delete frozen message
+///////////////////////////////////////////////////////////////////////////////
+    int             iLevel1 = atoi(ppszTokens[1]),
+                    iLevel2 = atoi(ppszTokens[2]);
+    char            szMessageFile[SYS_MAX_PATH] = "";
+
+    StrSNCpy(szMessageFile, ppszTokens[3]);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Get log file snapshot
+///////////////////////////////////////////////////////////////////////////////
+    char            szFileSS[SYS_MAX_PATH] = "";
+
+    SysGetTmpFile(szFileSS);
+
+    if (QueUtGetFrozenMsgFile(hSpoolQueue, iLevel1, iLevel2, szMessageFile, szFileSS) < 0)
+    {
+        ErrorPush();
+        CheckRemoveFile(szFileSS);
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_LISTFOLLOW_RESULT);
+
+
+///////////////////////////////////////////////////////////////////////////////
+//  Send client target file
+///////////////////////////////////////////////////////////////////////////////
+    if (MscSendTextFile(szFileSS, hBSock, pCTRLCfg->iTimeout) < 0)
+    {
+        ErrorPush();
+        SysRemove(szFileSS);
+        return (ErrorPop());
+    }
+
+    SysRemove(szFileSS);
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_etrn(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount < 2)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Do a matched flush of the rsnd arena
+///////////////////////////////////////////////////////////////////////////////
+    for (int ii = 1; ii < iTokensCount; ii++)
+    {
+        if (QueFlushRsndArena(hSpoolQueue, ppszTokens[ii]) < 0)
+        {
+            ErrorPush();
+            CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+            return (ErrorPop());
+        }
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, 0);
+
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_aliasdomainadd(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount != 3)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Filter params
+///////////////////////////////////////////////////////////////////////////////
+    char            szDomain[MAX_HOST_NAME] = "",
+                    szADomain[MAX_HOST_NAME] = "";
+
+    StrSNCpy(szDomain, ppszTokens[1]);
+    StrLower(szDomain);
+
+    StrSNCpy(szADomain, ppszTokens[2]);
+    StrLower(szADomain);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Target domain MUST exit ( alias of aliases are not permitted )
+///////////////////////////////////////////////////////////////////////////////
+    if (MDomLookupDomain(szDomain) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
+        return (ErrorPop());
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Add alias domain
+///////////////////////////////////////////////////////////////////////////////
+    if (ADomAddADomain(szADomain, szDomain) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, 0);
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_aliasdomaindel(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    if (iTokensCount != 2)
+    {
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ERR_BAD_CTRL_COMMAND);
+        ErrSetErrorCode(ERR_BAD_CTRL_COMMAND);
+        return (ERR_BAD_CTRL_COMMAND);
+    }
+
+///////////////////////////////////////////////////////////////////////////////
+//  Filter params
+///////////////////////////////////////////////////////////////////////////////
+    char            szADomain[MAX_HOST_NAME] = "";
+
+    StrSNCpy(szADomain, ppszTokens[1]);
+    StrLower(szADomain);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Remove alias domain
+///////////////////////////////////////////////////////////////////////////////
+    if (ADomRemoveADomain(szADomain) < 0)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrorFetch());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, 0);
+
+    return (0);
+
+}
+
+
+
+static int      CTRLDo_aliasdomainlist(CTRLConfig * pCTRLCfg, BSOCK_HANDLE hBSock,
+                        char const * const * ppszTokens, int iTokensCount)
+{
+
+    ADOMAIN_HANDLE  hADomainDB = ADomOpenDB();
+
+    if (hADomainDB == INVALID_ADOMAIN_HANDLE)
+    {
+        ErrorPush();
+        CTRLSendCmdResult(pCTRLCfg, hBSock, ErrGetErrorCode());
+        return (ErrorPop());
+    }
+
+
+    CTRLSendCmdResult(pCTRLCfg, hBSock, CTRL_LISTFOLLOW_RESULT);
+
+
+    char const * const *ppszStrings = ADomGetFirstDomain(hADomainDB);
+
+    for (; ppszStrings != NULL; ppszStrings = ADomGetNextDomain(hADomainDB))
+    {
+        if ((iTokensCount < 2) || StrStringsRIWMatch(&ppszTokens[1], ppszStrings[adomADomain]))
+        {
+            if (BSckVSendString(hBSock, pCTRLCfg->iTimeout, "\"%s\"\t\"%s\"",
+                    ppszStrings[adomADomain], ppszStrings[adomDomain]) < 0)
+            {
+                ErrorPush();
+                ADomCloseDB(hADomainDB);
+                return (ErrorPop());
+            }
+        }
+    }
+
+    ADomCloseDB(hADomainDB);
+
+    BSckSendString(hBSock, ".", pCTRLCfg->iTimeout);
+
+    return (0);
+
+}
