@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "BuffSock.h"
 #include "SList.h"
 #include "MailConfig.h"
+#include "MessQueue.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
 #include "SvrUtils.h"
@@ -44,6 +45,7 @@
 #define DNS_MAX_RESP_PACKET     1024
 #define DNS_SEND_RETRIES        3
 #define DNS_MAX_RR_DATA         256
+#define DNS_RESPDATA_EXTRA      (2 * sizeof(int))
 
 #if defined(LITTLE_ENDIAN_CPU)
 #define DNS_LABEL_LEN_MASK      0xff3f
@@ -70,12 +72,12 @@ struct DNSQuery
 
 struct DNSResourceRecord
 {
-    char            szName[256];
+    char            szName[MAX_HOST_NAME];
     SYS_UINT16      Type;
     SYS_UINT16      Class;
     SYS_UINT32      TTL;
     SYS_UINT16      Lenght;
-    SYS_UINT8       RespData[DNS_MAX_RR_DATA];
+    SYS_UINT8 const     *pRespData;
 };
 
 struct DNSNameNode
@@ -90,18 +92,21 @@ struct DNSNameNode
 
 
 
+static SYS_UINT8   *DNS_AllocRespData(int iSize);
+static int      DNS_FreeRespData(SYS_UINT8 * pRespData);
+static int      DNS_RespDataSize(SYS_UINT8 const * pRespData);
 static DNSNameNode *DNS_AllocNameNode(char const * pszServer, char const * pszQuery);
 static void     DNS_FreeNameNode(DNSNameNode * pDNSNN);
 static void     DNS_FreeNameList(HSLIST & hNameList);
 static DNSNameNode *DNS_GetNameNode(HSLIST & hNameList, char const * pszServer, char const * pszQuery);
 static int      DNS_AddNameNode(HSLIST & hNameList, char const * pszServer, char const * pszQuery);
 static int      DNS_GetResourceRecord(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
-                        DNSResourceRecord * pRR = NULL);
+                        DNSResourceRecord * pRR = NULL, int * piRRLength = NULL);
 static int      DNS_GetName(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
-                        char *pszInetName = NULL);
+                        char *pszInetName = NULL, int * piRRLength = NULL);
 static int      DNS_GetQuery(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
                         char *pszInetName = NULL, SYS_UINT16 * pType = NULL,
-                        SYS_UINT16 * pClass = NULL);
+                        SYS_UINT16 * pClass = NULL, int * piRRLength = NULL);
 static int      DNS_NameCopy(SYS_UINT8 * pDNSQName, char const * pszInetName);
 static SYS_UINT16 DNS_GetUniqueQueryId(void);
 static int      DNS_RequestSetup(DNSQuery & DNSQ, unsigned int uOpCode,
@@ -126,7 +131,7 @@ static int      DNS_QueryDomainMX(char const * pszDNSServer, char const * pszDom
                         char *&pszMXDomains, SYS_UINT32 * pTTL = NULL);
 static int      DNS_GetNameServersLL(char const * pszDNSServer, char const * pszDomain,
                         char const * pszRespFile, HSLIST & hNameList, SYS_UINT32 * pTTL = NULL);
-static char    *DNS_GetRootsFile(char *pszRootsFilePath);
+static char    *DNS_GetRootsFile(char *pszRootsFilePath, int iMaxPath);
 
 
 
@@ -134,6 +139,50 @@ static char    *DNS_GetRootsFile(char *pszRootsFilePath);
 
 
 
+static SYS_UINT8   *DNS_AllocRespData(int iSize)
+{
+
+    int             iSizeExtra = DNS_RESPDATA_EXTRA;
+    SYS_UINT8      *pBaseData = (SYS_UINT8 *) SysAlloc(iSize + iSizeExtra);
+
+    if (pBaseData == NULL)
+        return (NULL);
+
+
+    ((int *) pBaseData)[0] = iSize;
+
+
+    return (pBaseData + iSizeExtra);
+
+}
+
+
+
+
+static int          DNS_FreeRespData(SYS_UINT8 * pRespData)
+{
+
+    SYS_UINT8      *pBaseData = pRespData - DNS_RESPDATA_EXTRA;
+
+    SysFree(pBaseData);
+
+    return (0);
+
+}
+
+
+
+
+static int          DNS_RespDataSize(SYS_UINT8 const * pRespData)
+{
+
+    SYS_UINT8 const     *pBaseData = pRespData - DNS_RESPDATA_EXTRA;
+
+    return (((int *) pBaseData)[0]);
+
+}
+
+
 
 
 static DNSNameNode *DNS_AllocNameNode(char const * pszServer, char const * pszQuery)
@@ -227,7 +276,7 @@ static int      DNS_AddNameNode(HSLIST & hNameList, char const * pszServer, char
 
 
 static int      DNS_GetResourceRecord(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
-                        DNSResourceRecord * pRR)
+                        DNSResourceRecord * pRR, int * piRRLength)
 {
 
     if (pRR != NULL)
@@ -236,8 +285,12 @@ static int      DNS_GetResourceRecord(SYS_UINT8 const * pBaseData, SYS_UINT8 con
 ///////////////////////////////////////////////////////////////////////////////
 //  Read name field
 ///////////////////////////////////////////////////////////////////////////////
+    int             iRRLen = 0;
     char           *pszName = (pRR != NULL) ? pRR->szName : NULL;
-    int             iRRLen = DNS_GetName(pBaseData, pRespData, pszName);
+
+    if (DNS_GetName(pBaseData, pRespData, pszName, &iRRLen) < 0)
+        return (ErrGetErrorCode());
+
 
     pRespData += iRRLen;
 
@@ -283,70 +336,116 @@ static int      DNS_GetResourceRecord(SYS_UINT8 const * pBaseData, SYS_UINT8 con
 //  Read RR data
 ///////////////////////////////////////////////////////////////////////////////
     if (pRR != NULL)
-        memcpy(pRR->RespData, pRespData, Lenght);
+        pRR->pRespData = pRespData;
 
     iRRLen += (int) Lenght;
 
 
-    return (iRRLen);
+    if (piRRLength != NULL)
+        *piRRLength = iRRLen;
+
+
+    return (0);
 
 }
 
 
 
 static int      DNS_GetName(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
-                        char *pszInetName)
+                        char * pszInetName, int * piRRLength)
 {
 
-    char            szNameBuffer[512] = "";
+    char            szNameBuffer[MAX_HOST_NAME] = "";
 
     if (pszInetName == NULL)
         pszInetName = szNameBuffer;
 
-    int             iNameLen = 0,
+    int             iBaseLength = DNS_RespDataSize(pBaseData),
+                    iCurrOffset = (int) (pRespData - pBaseData),
+                    iDataLength = 0,
+                    iNameLength = 0,
                     iBackLink = 0;
 
     while (*pRespData != 0)
     {
         if (*pRespData & DNS_LABEL_LEN_INVMASK)
         {
-            int             iLabelOffset = (int) ntohs(MscReadUint16(pRespData) & DNS_LABEL_LEN_MASK);
+///////////////////////////////////////////////////////////////////////////////
+//  Got displacement from base-data ( boundary checked )
+///////////////////////////////////////////////////////////////////////////////
+            if ((iCurrOffset = (int)
+                    ntohs(MscReadUint16(pRespData) & DNS_LABEL_LEN_MASK)) >= iBaseLength)
+            {
+                ErrSetErrorCode(ERR_BAD_DNS_NAME_RECORD);
+                return (ERR_BAD_DNS_NAME_RECORD);
+            }
 
-            pRespData = pBaseData + iLabelOffset;
+            pRespData = pBaseData + iCurrOffset;
 
             if (!iBackLink)
-                iNameLen += sizeof(SYS_UINT16), ++iBackLink;
+                iDataLength += sizeof(SYS_UINT16), ++iBackLink;
 
             continue;
         }
 
-        int             iLabelLen = (int) *pRespData;
+///////////////////////////////////////////////////////////////////////////////
+//  Extract label length ( boundary checked )
+///////////////////////////////////////////////////////////////////////////////
+        int             iLabelLength = (int) *pRespData;
 
-        memcpy(pszInetName, pRespData + 1, iLabelLen);
-        pszInetName[iLabelLen] = '.';
+        if (((iNameLength + iLabelLength + 2) >= MAX_HOST_NAME) ||
+                ((iCurrOffset + iLabelLength + 1) >= iBaseLength))
+        {
+            ErrSetErrorCode(ERR_BAD_DNS_NAME_RECORD);
+            return (ERR_BAD_DNS_NAME_RECORD);
+        }
 
-        pszInetName += iLabelLen + 1;
+///////////////////////////////////////////////////////////////////////////////
+//  Append to name and update pointers
+///////////////////////////////////////////////////////////////////////////////
+        memcpy(pszInetName, pRespData + 1, iLabelLength);
+        pszInetName[iLabelLength] = '.';
+
+        iNameLength += iLabelLength + 1;
+        pszInetName += iLabelLength + 1;
+
+///////////////////////////////////////////////////////////////////////////////
+//  If we've not got a back-jump, update the data length
+///////////////////////////////////////////////////////////////////////////////
         if (!iBackLink)
-            iNameLen += iLabelLen + 1;
-        pRespData += iLabelLen + 1;
+            iDataLength += iLabelLength + 1;
+
+///////////////////////////////////////////////////////////////////////////////
+//  Move pointers
+///////////////////////////////////////////////////////////////////////////////
+        iCurrOffset += iLabelLength + 1;
+        pRespData += iLabelLength + 1;
     }
 
     *pszInetName = '\0';
 
-    return ((!iBackLink) ? iNameLen + 1 : iNameLen);
+    if (piRRLength != NULL)
+        *piRRLength = (!iBackLink) ? iDataLength + 1 : iDataLength;
+
+    return (0);
 
 }
 
 
 
 static int      DNS_GetQuery(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRespData,
-                        char *pszInetName, SYS_UINT16 * pType, SYS_UINT16 * pClass)
+                        char *pszInetName, SYS_UINT16 * pType, SYS_UINT16 * pClass,
+                        int * piRRLength)
 {
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Read name field
 ///////////////////////////////////////////////////////////////////////////////
-    int             iQueryLen = DNS_GetName(pBaseData, pRespData, pszInetName);
+    int             iQueryLen = 0;
+
+    if (DNS_GetName(pBaseData, pRespData, pszInetName, &iQueryLen) < 0)
+        return (ErrGetErrorCode());
+
 
     pRespData += iQueryLen;
 
@@ -368,7 +467,12 @@ static int      DNS_GetQuery(SYS_UINT8 const * pBaseData, SYS_UINT8 const * pRes
     pRespData += sizeof(SYS_UINT16);
     iQueryLen += sizeof(SYS_UINT16);
 
-    return (iQueryLen);
+
+    if (piRRLength != NULL)
+        *piRRLength = iQueryLen;
+
+
+    return (0);
 
 }
 
@@ -415,9 +519,17 @@ static int      DNS_NameCopy(SYS_UINT8 * pDNSQName, char const * pszInetName)
 static SYS_UINT16 DNS_GetUniqueQueryId(void)
 {
 
-    SYS_UINT32      uThreadId = (SYS_UINT32) SysGetCurrentThreadId();
+    static SYS_SPINLOCK IDSpin = SYS_SPINLOCK_UNLOCKED;
+    static SYS_UINT16   uDnsQueryId = 0;
+    SYS_UINT16      uQueryId;
 
-    return ((SYS_UINT16) uThreadId);
+    SysSpinAcquire(&IDSpin);
+
+    uQueryId = ++uDnsQueryId;
+
+    SysSpinRelease(&IDSpin);
+
+    return (uQueryId);
 
 }
 
@@ -555,7 +667,7 @@ static SYS_UINT8 *DNS_QuerySendStream(char const * pszDNSServer, int iPortNo, in
 
     int             iPacketLenght = (int) ntohs(QLenght);
 
-    SYS_UINT8      *pRespData = (SYS_UINT8 *) SysAlloc(iPacketLenght + 1);
+    SYS_UINT8      *pRespData = DNS_AllocRespData(iPacketLenght + 1);
 
     if (pRespData == NULL)
     {
@@ -571,7 +683,7 @@ static SYS_UINT8 *DNS_QuerySendStream(char const * pszDNSServer, int iPortNo, in
     if (SysRecv(SockFD, (char *) pRespData, iPacketLenght, iTimeout) != iPacketLenght)
     {
         ErrorPush();
-        SysFree(pRespData);
+        DNS_FreeRespData(pRespData);
         SysCloseSocket(SockFD);
         ErrSetErrorCode(ErrorFetch());
         return (NULL);
@@ -584,7 +696,7 @@ static SYS_UINT8 *DNS_QuerySendStream(char const * pszDNSServer, int iPortNo, in
 
     if (DNSQ.DNSH.RD && !pDNSH->RA)
     {
-        SysFree(pRespData);
+        DNS_FreeRespData(pRespData);
         ErrSetErrorCode(ERR_DNS_RECURSION_NOT_AVAILABLE);
         return (NULL);
     }
@@ -632,6 +744,7 @@ static SYS_UINT8 *DNS_QuerySendDGram(char const * pszDNSServer, int iPortNo, int
 
         ZeroData(RecvAddr);
 
+
         int             iPacketLenght = SysRecvDataFrom(SockFD, (struct sockaddr *) & RecvAddr, sizeof(RecvAddr),
                 (char *) RespBuffer, sizeof(RespBuffer), iTimeout);
 
@@ -639,6 +752,7 @@ static SYS_UINT8 *DNS_QuerySendDGram(char const * pszDNSServer, int iPortNo, int
         if ((iPacketLenght < 0) || (iPacketLenght < sizeof(DNS_HEADER)))
             continue;
 
+
         DNS_HEADER     *pDNSH = (DNS_HEADER *) RespBuffer;
 
         if (pDNSH->Id != DNSQ.DNSH.Id)
@@ -660,7 +774,7 @@ static SYS_UINT8 *DNS_QuerySendDGram(char const * pszDNSServer, int iPortNo, int
             return (NULL);
         }
 
-        SYS_UINT8      *pRespData = (SYS_UINT8 *) SysAlloc(iPacketLenght + 1);
+        SYS_UINT8      *pRespData = DNS_AllocRespData(iPacketLenght + 1);
 
         if (pRespData == NULL)
         {
@@ -713,7 +827,7 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszDoma
     {
         if ((pMXFile = fopen(pszRespFile, "wb")) == NULL)
         {
-            ErrSetErrorCode(ERR_FILE_CREATE);
+            ErrSetErrorCode(ERR_FILE_CREATE, pszRespFile);
             return (ERR_FILE_CREATE);
         }
     }
@@ -728,12 +842,18 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszDoma
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.QDCount; ii++)
     {
+        int             iQLenght = 0;
         SYS_UINT16      Type = 0,
                         Class = 0;
         char            szInetName[MAX_HOST_NAME] = "";
 
-
-        int             iQLenght = DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class);
+        if (DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class, &iQLenght) < 0)
+        {
+            ErrorPush();
+            if (pMXFile != NULL)
+                fclose(pMXFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iQLenght;
@@ -746,23 +866,35 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszDoma
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.ANCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+        {
+            ErrorPush();
+            if (pMXFile != NULL)
+                fclose(pMXFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iRRLenght;
 
 
-        SYS_UINT8      *pMXData = RR.RespData;
+        SYS_UINT8 const     *pMXData = RR.pRespData;
         SYS_UINT16      Preference = ntohs(MscReadUint16(pMXData));
 
         pMXData += sizeof(SYS_UINT16);
 
         char            szMXDomain[MAX_HOST_NAME] = "";
 
-        DNS_GetName(pBaseData, pMXData, szMXDomain);
+        if (DNS_GetName(pBaseData, pMXData, szMXDomain) < 0)
+        {
+            ErrorPush();
+            if (pMXFile != NULL)
+                fclose(pMXFile);
+            return (ErrorPop());
+        }
 
         if (ii == 0)
             fprintf(pMXFile, "%d:%s", (int) Preference, szMXDomain);
@@ -789,20 +921,22 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszDoma
 ///////////////////////////////////////////////////////////////////////////////
     for (ii = 0; ii < (int) pDNSQ->DNSH.NSCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+            return (ErrGetErrorCode());
 
 
         pRespData += iRRLenght;
 
 
-        SYS_UINT8      *pNSData = RR.RespData;
+        SYS_UINT8 const     *pNSData = RR.pRespData;
 
         char            szNSName[MAX_HOST_NAME] = "";
 
-        DNS_GetName(pBaseData, pNSData, szNSName);
+        if (DNS_GetName(pBaseData, pNSData, szNSName) < 0)
+            return (ErrGetErrorCode());
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Recursively try authority name servers
@@ -818,10 +952,11 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszDoma
 ///////////////////////////////////////////////////////////////////////////////
     for (ii = 0; ii < (int) pDNSQ->DNSH.ARCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+            return (ErrGetErrorCode());
 
 
         pRespData += iRRLenght;
@@ -879,12 +1014,17 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszResp
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.QDCount; ii++)
     {
+        int             iQLenght = 0;
         SYS_UINT16      Type = 0,
                         Class = 0;
         char            szInetName[MAX_HOST_NAME] = "";
 
-
-        int             iQLenght = DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class);
+        if (DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class, &iQLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pMXFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iQLenght;
@@ -898,23 +1038,33 @@ static int      DNS_DecodeResponseMX(SYS_UINT8 * pRespData, char const * pszResp
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.ANCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pMXFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iRRLenght;
 
 
-        SYS_UINT8      *pMXData = RR.RespData;
+        SYS_UINT8 const     *pMXData = RR.pRespData;
         SYS_UINT16      Preference = ntohs(MscReadUint16(pMXData));
 
         pMXData += sizeof(SYS_UINT16);
 
         char            szMXDomain[MAX_HOST_NAME] = "";
 
-        DNS_GetName(pBaseData, pMXData, szMXDomain);
+        if (DNS_GetName(pBaseData, pMXData, szMXDomain) < 0)
+        {
+            ErrorPush();
+            fclose(pMXFile);
+            return (ErrorPop());
+        }
 
         if (ii == 0)
             fprintf(pMXFile, "%d:%s", (int) Preference, szMXDomain);
@@ -957,7 +1107,7 @@ static int      DNS_DecodeResponseNS(SYS_UINT8 * pRespData, char const * pszResp
 
     if (pNSFile == NULL)
     {
-        ErrSetErrorCode(ERR_FILE_CREATE);
+        ErrSetErrorCode(ERR_FILE_CREATE, pszRespFile);
         return (ERR_FILE_CREATE);
     }
 
@@ -977,12 +1127,17 @@ static int      DNS_DecodeResponseNS(SYS_UINT8 * pRespData, char const * pszResp
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.QDCount; ii++)
     {
+        int             iQLenght = 0;
         SYS_UINT16      Type = 0,
                         Class = 0;
         char            szInetName[MAX_HOST_NAME] = "";
 
-
-        int             iQLenght = DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class);
+        if (DNS_GetQuery(pBaseData, pRespData, szInetName, &Type, &Class, &iQLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iQLenght;
@@ -996,20 +1151,30 @@ static int      DNS_DecodeResponseNS(SYS_UINT8 * pRespData, char const * pszResp
 
     for (ii = 0; ii < (int) pDNSQ->DNSH.ANCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iRRLenght;
 
 
-        SYS_UINT8      *pNSData = RR.RespData;
+        SYS_UINT8 const     *pNSData = RR.pRespData;
 
         char            szNSName[MAX_HOST_NAME] = "";
 
-        DNS_GetName(pBaseData, pNSData, szNSName);
+        if (DNS_GetName(pBaseData, pNSData, szNSName) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
         fprintf(pNSFile, "%s\n", szNSName);
 
@@ -1036,20 +1201,30 @@ static int      DNS_DecodeResponseNS(SYS_UINT8 * pRespData, char const * pszResp
 ///////////////////////////////////////////////////////////////////////////////
     for (ii = 0; ii < (int) pDNSQ->DNSH.NSCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iRRLenght;
 
 
-        SYS_UINT8      *pNSData = RR.RespData;
+        SYS_UINT8 const     *pNSData = RR.pRespData;
 
         char            szNSName[MAX_HOST_NAME] = "";
 
-        DNS_GetName(pBaseData, pNSData, szNSName);
+        if (DNS_GetName(pBaseData, pNSData, szNSName) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
         fprintf(pNSFile, "%s\n", szNSName);
 
@@ -1076,10 +1251,15 @@ static int      DNS_DecodeResponseNS(SYS_UINT8 * pRespData, char const * pszResp
 ///////////////////////////////////////////////////////////////////////////////
     for (ii = 0; ii < (int) pDNSQ->DNSH.ARCount; ii++)
     {
+        int             iRRLenght = 0;
         DNSResourceRecord RR;
 
-
-        int             iRRLenght = DNS_GetResourceRecord(pBaseData, pRespData, &RR);
+        if (DNS_GetResourceRecord(pBaseData, pRespData, &RR, &iRRLenght) < 0)
+        {
+            ErrorPush();
+            fclose(pNSFile);
+            return (ErrorPop());
+        }
 
 
         pRespData += iRRLenght;
@@ -1121,7 +1301,7 @@ static int      DNS_FindDomainMX(char const * pszDNSServer, char const * pszDoma
             hNameList, pszRespFile, pTTL);
 
 
-    SysFree(pRespData);
+    DNS_FreeRespData(pRespData);
 
     return (iDecodeResult);
 
@@ -1205,13 +1385,13 @@ int             DNS_QueryNameServers(char const * pszDNSServer, char const * psz
     if (DNS_DecodeResponseNS(pRespData, pszRespFile, bAuth, pTTL) < 0)
     {
         ErrorPush();
-        SysFree(pRespData);
+        DNS_FreeRespData(pRespData);
 
         return (ErrorPop());
     }
 
 
-    SysFree(pRespData);
+    DNS_FreeRespData(pRespData);
 
     return (0);
 
@@ -1326,8 +1506,8 @@ int             DNS_DomainNameServers(char const * pszDomain, char const * pszRe
     char            szRootsFile[SYS_MAX_PATH] = "",
                     szRespFile[SYS_MAX_PATH] = "";
 
-    DNS_GetRootsFile(szRootsFile);
-    strcpy(szRespFile, szRootsFile);
+    DNS_GetRootsFile(szRootsFile, sizeof(szRootsFile));
+    StrSNCpy(szRespFile, szRootsFile);
 
 
     char          **ppszDomains = StrTokenize(pszDomain, ". \t");
@@ -1343,7 +1523,7 @@ int             DNS_DomainNameServers(char const * pszDomain, char const * pszRe
         char            szCurrDomain[MAX_HOST_NAME] = "";
 
         sprintf(szCurrDomain, "%s.%s", ppszDomains[iSubDomains], szPrevDomain);
-        strcpy(szPrevDomain, szCurrDomain);
+        StrSNCpy(szPrevDomain, szCurrDomain);
 
 
         FILE           *pNSFile = fopen(szRespFile, "rt");
@@ -1386,7 +1566,7 @@ int             DNS_DomainNameServers(char const * pszDomain, char const * pszRe
         if (strcmp(szRespFile, szRootsFile) != 0)
             SysRemove(szRespFile);
 
-        strcpy(szRespFile, szRespFile2);
+        StrSNCpy(szRespFile, szRespFile2);
     }
 
     StrFreeStrings(ppszDomains);
@@ -1410,12 +1590,12 @@ int             DNS_DomainNameServers(char const * pszDomain, char const * pszRe
 
 
 
-static char    *DNS_GetRootsFile(char *pszRootsFilePath)
+static char    *DNS_GetRootsFile(char *pszRootsFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszRootsFilePath);
+    CfgGetRootPath(pszRootsFilePath, iMaxPath);
 
-    strcat(pszRootsFilePath, ROOTS_FILE);
+    StrNCat(pszRootsFilePath, ROOTS_FILE, iMaxPath);
 
     return (pszRootsFilePath);
 
@@ -1541,13 +1721,13 @@ int             DNS_GetDomainMXDirect(char const * pszDNSServer, char const * ps
     if (DNS_DecodeResponseMX(pRespData, szRespFile, pTTL) < 0)
     {
         ErrorPush();
-        SysFree(pRespData);
+        DNS_FreeRespData(pRespData);
         CheckRemoveFile(szRespFile);
 
         return (ErrorPop());
     }
 
-    SysFree(pRespData);
+    DNS_FreeRespData(pRespData);
 
 
     FILE           *pMXFile = fopen(szRespFile, "rb");
