@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -75,7 +75,6 @@
 #define ERR_LINK_EXIST              (-44)
 #define ERR_WRITE_LINKS_FILE        (-45)
 #define ERR_LINK_NOT_FOUND          (-46)
-#define ERR_NO_SMTP_SPOOL_FILES     (-47)
 #define ERR_SMTPGW_FILE_NOT_FOUND   (-48)
 #define ERR_GATEWAY_ALREADY_EXIST   (-49)
 #define ERR_GATEWAY_NOT_FOUND       (-50)
@@ -85,7 +84,6 @@
 #define ERR_MLUSER_NOT_FOUND        (-54)
 #define ERR_SPOOL_FILE_NOT_FOUND    (-55)
 #define ERR_INVALID_SPOOL_FILE      (-56)
-#define ERR_SPOOL_FILE_NOT_READY    (-57)
 #define ERR_SPOOL_FILE_EXPIRED      (-58)
 #define ERR_SMTPRELAY_FILE_NOT_FOUND (-59)
 #define ERR_DOMAINS_FILE_NOT_FOUND  (-60)
@@ -137,7 +135,6 @@
 #define ERR_DOMAIN_ALREADY_HANDLED  (-109)
 #define ERR_BAD_CTRL_LOGIN          (-110)
 #define ERR_CTRL_ACCOUNTS_FILE_NOT_FOUND    (-111)
-#define ERR_RBL_SPAMMER             (-112)
 #define ERR_SPAMMER_IP              (-113)
 #define ERR_TRUNCATED_DGRAM_DNS_RESPONSE    (-114)
 #define ERR_NO_DGRAM_DNS_RESPONSE   (-115)
@@ -150,7 +147,6 @@
 #define ERR_NO_MAILPROC_FILE        (-123)
 #define ERR_DNS_RECURSION_NOT_AVAILABLE (-124)
 #define ERR_POP3_EXTERNAL_LINK_DISABLED (-125)
-#define ERR_RSS_SPAMMER             (-126)
 #define ERR_BAD_DOMAIN_PROC_CMD_SYNTAX  (-128)
 #define ERR_NOT_A_CUSTOM_DOMAIN     (-129)
 #define ERR_NO_MORE_TOKENS          (-130)
@@ -207,9 +203,6 @@
 #define ERR_SMTPFWD_NOT_FOUND       (-182)
 #define ERR_USER_BREAK              (-183)
 #define ERR_SET_THREAD_PRIORITY     (-184)
-#define ERR_INVALID_QUEUE_PATH      (-185)
-#define ERR_QUEUE_ARRAY_FULL        (-186)
-#define ERR_QUEUE_ENTRY_NOT_FOUND   (-187)
 #define ERR_NULL_SENDER             (-188)
 #define ERR_RCPTTO_UNKNOWN          (-189)
 #define ERR_LOADMODULE              (-190)
@@ -223,13 +216,33 @@
 #define ERR_THREADCREATE            (-198)
 #define ERR_CREATEMUTEX             (-199)
 #define ERR_NO_LOCAL_SPOOL_FILES    (-200)
-#define ERR_NO_POP3_DOMAIN          (-201)
+#define ERR_NO_HANDLED_DOMAIN       (-201)
 #define ERR_INVALID_MAIL_DOMAIN     (-202)
 #define ERR_BAD_CMDSTR_CHARS        (-203)
 #define ERR_FETCHMSG_UNDELIVERED    (-204)
-
-
-
+#define ERR_USER_VAR_NOT_FOUND      (-205)
+#define ERR_NO_POP3_IP              (-208)
+#define ERR_NO_MESSAGE_FILE         (-209)
+#define ERR_GET_DISK_SPACE_INFO     (-210)
+#define ERR_GET_MEMORY_INFO         (-211)
+#define ERR_LOW_DISK_SPACE          (-212)
+#define ERR_LOW_VM_SPACE            (-213)
+#define ERR_USER_DISABLED           (-214)
+#define ERR_BAD_DNS_NAME_RECORD     (-215)
+#define ERR_MESSAGE_SIZE            (-216)
+#define ERR_SMTPSRV_MSG_SIZE        (-217)
+#define ERR_MAPS_CONTAINED          (-218)
+#define ERR_ADOMAIN_FILE_NOT_FOUND  (-219)
+#define ERR_ADOMAIN_EXIST           (-220)
+#define ERR_ADOMAIN_NOT_FOUND       (-221)
+#define ERR_NOT_A_CMD_ALIAS         (-222)
+#define ERR_GETSOCKOPT              (-223)
+#define ERR_NO_HDR_FETCH_TAGS       (-224)
+#define ERR_SET_FILE_TIME           (-225)
+#define ERR_LISTDIR_NOT_FOUND       (-226)
+#define ERR_DUPLICATE_HANDLE        (-227)
+#define ERR_EMPTY_LOG               (-228)
+#define ERR_BAD_RELATIVE_PATH       (-229)
 
 
 
