@@ -1,6 +1,6 @@
 /*
- *  MailSvr by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999  Davide Libenzi
+ *  XMail by Davide Libenzi ( Intranet and Internet mail server )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Davide Libenzi <davide_libenzi@mycio.com>
+ *  Davide Libenzi <davidel@xmailserver.org>
  *
  */
 
@@ -30,6 +30,7 @@
 #include "SList.h"
 #include "BuffSock.h"
 #include "MailConfig.h"
+#include "MessQueue.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
 #include "SvrUtils.h"
@@ -74,7 +75,7 @@ struct ExAlDBScanData
 
 
 static int      ExAlRebuildAliasIndexes(char const * pszAliasFilePath);
-static char    *ExAlGetTableFilePath(char *pszLnkFilePath);
+static char    *ExAlGetTableFilePath(char *pszLnkFilePath, int iMaxPath);
 static ExtAlias *ExAlGetAliasFromStrings(char **ppszStrings);
 static int      ExAlWriteAlias(FILE * pAliasFile, ExtAlias * pExtAlias);
 
@@ -105,7 +106,7 @@ int             ExAlCheckAliasIndexes(void)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Align RmtDomain-RmtName index
@@ -137,12 +138,12 @@ static int      ExAlRebuildAliasIndexes(char const * pszAliasFilePath)
 
 
 
-static char    *ExAlGetTableFilePath(char *pszLnkFilePath)
+static char    *ExAlGetTableFilePath(char *pszLnkFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszLnkFilePath);
+    CfgGetRootPath(pszLnkFilePath, iMaxPath);
 
-    strcat(pszLnkFilePath, SVR_EXT_ALIAS_FILE);
+    StrNCat(pszLnkFilePath, SVR_EXT_ALIAS_FILE, iMaxPath);
 
     return (pszLnkFilePath);
 
@@ -277,11 +278,12 @@ int             ExAlAddAlias(ExtAlias * pExtAlias)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -360,11 +362,12 @@ ExtAlias       *ExAlGetAlias(char const * pszRmtDomain, char const * pszRmtName)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (NULL);
@@ -405,7 +408,7 @@ int             ExAlRemoveAlias(ExtAlias * pExtAlias)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -413,7 +416,8 @@ int             ExAlRemoveAlias(ExtAlias * pExtAlias)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -521,7 +525,7 @@ int             ExAlRemoveUserAliases(const char *pszDomain, const char *pszName
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -529,7 +533,8 @@ int             ExAlRemoveUserAliases(const char *pszDomain, const char *pszName
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -642,7 +647,7 @@ int             ExAlRemoveDomainAliases(const char *pszDomain)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -650,7 +655,8 @@ int             ExAlRemoveDomainAliases(const char *pszDomain)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -765,11 +771,12 @@ int             ExAlGetDBFileSnapShot(const char *pszFileName)
 
     char            szAliasFilePath[SYS_MAX_PATH] = "";
 
-    ExAlGetTableFilePath(szAliasFilePath);
+    ExAlGetTableFilePath(szAliasFilePath, sizeof(szAliasFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAliasFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szAliasFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
