@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "MailConfig.h"
 #include "UsrUtils.h"
 #include "SvrUtils.h"
+#include "MessQueue.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
 #include "Maildir.h"
@@ -55,10 +56,10 @@ int             MdirCreateStructure(char const * pszBasePath)
 ///////////////////////////////////////////////////////////////////////////////
     char            szMaildirPath[SYS_MAX_PATH] = "";
 
-    strcpy(szMaildirPath, pszBasePath);
+    StrSNCpy(szMaildirPath, pszBasePath);
 
     AppendSlash(szMaildirPath);
-    strcat(szMaildirPath, MAILDIR_DIRECTORY);
+    StrSNCat(szMaildirPath, MAILDIR_DIRECTORY);
 
     if (SysMakeDir(szMaildirPath) < 0)
         return (ErrGetErrorCode());
@@ -134,9 +135,9 @@ int             MdirMoveTmpEntryInNew(char const * pszTmpEntryPath)
                    *pszSlash = strrchr(pszTmpEntryPath, SYS_SLASH_CHAR);
     char            szNewEntryPath[SYS_MAX_PATH] = "";
 
-    strcpy(szNewEntryPath, pszTmpEntryPath);
-    strcpy(szNewEntryPath + iBaseLength, pszNewDir);
-    strcat(szNewEntryPath, pszSlash + 1);
+    StrSNCpy(szNewEntryPath, pszTmpEntryPath);
+    StrNCpy(szNewEntryPath + iBaseLength, pszNewDir, sizeof(szNewEntryPath) - iBaseLength);
+    StrNCat(szNewEntryPath, pszSlash + 1, sizeof(szNewEntryPath));
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Move to Maildir/new
