@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,13 @@
 #include "SysInclude.h"
 #include "SysDep.h"
 #include "SvrDefines.h"
+#include "SList.h"
 #include "ShBlocks.h"
+#include "UsrUtils.h"
+#include "SvrUtils.h"
+#include "MessQueue.h"
+#include "SMAILUtils.h"
+#include "QueueUtils.h"
 #include "AppDefines.h"
 #include "MailSvr.h"
 
@@ -36,6 +42,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 #define SERVICE
 
+#define NULFILE         "nul"
 
 
 
@@ -78,7 +85,7 @@ int             main(int iArgCount, char *pszArgs[])
 
 
 
-
+static int      MnSetupStdHandles(void);
 static VOID WINAPI ServiceCtrl(DWORD dwCtrlCode);
 static VOID WINAPI ServiceMain(DWORD dwArgc, LPTSTR lpszArgv[]);
 static BOOL     CmdInstallService(DWORD dwStartType);
@@ -104,6 +111,56 @@ static TCHAR    szErr[2048] = _T("");
 
 
 
+static int      MnSetupStdHandles(void)
+{
+
+    HANDLE      hInFile = CreateFile(NULFILE, GENERIC_READ | GENERIC_WRITE,
+            FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL, OPEN_EXISTING, 0, NULL);
+
+    if (hInFile == INVALID_HANDLE_VALUE)
+    {
+        AddToMessageLog(_T("CreateFile"));
+        return (-1);
+    }
+
+    HANDLE      hOutFile = CreateFile(NULFILE, GENERIC_READ | GENERIC_WRITE,
+            FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL, OPEN_EXISTING, 0, NULL);
+
+    if (hOutFile == INVALID_HANDLE_VALUE)
+    {
+        AddToMessageLog(_T("CreateFile"));
+        CloseHandle(hInFile);
+        return (-1);
+    }
+
+    HANDLE      hErrFile = CreateFile(NULFILE, GENERIC_READ | GENERIC_WRITE,
+            FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL, OPEN_EXISTING, 0, NULL);
+
+    if (hErrFile == INVALID_HANDLE_VALUE)
+    {
+        AddToMessageLog(_T("CreateFile"));
+        CloseHandle(hOutFile);
+        CloseHandle(hInFile);
+        return (-1);
+    }
+
+    if (!SetStdHandle(STD_INPUT_HANDLE, hInFile) || !SetStdHandle(STD_OUTPUT_HANDLE, hOutFile) ||
+            !SetStdHandle(STD_ERROR_HANDLE, hErrFile))
+    {
+        AddToMessageLog(_T("SetStdHandle"));
+        CloseHandle(hErrFile);
+        CloseHandle(hOutFile);
+        CloseHandle(hInFile);
+        return (-1);
+    }
+
+    return (0);
+
+}
+
 
 
 int             _tmain(int argc, TCHAR * argv[])
@@ -149,7 +206,15 @@ int             _tmain(int argc, TCHAR * argv[])
     _tprintf(_T("This may take several seconds.  Please wait.\n"));
 
 
+///////////////////////////////////////////////////////////////////////////////
+//  Setup std handles
+///////////////////////////////////////////////////////////////////////////////
+    if (MnSetupStdHandles() < 0)
+        return (1);
 
+///////////////////////////////////////////////////////////////////////////////
+//  Service loop
+///////////////////////////////////////////////////////////////////////////////
     if (!StartServiceCtrlDispatcher(DispTable))
         AddToMessageLog(_T("StartServiceCtrlDispatcher"));
 
