@@ -1,5 +1,24 @@
-
-
+/*
+ *  MkUsers by Davide Libenzi ( bulk mail users generator for XMail )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ *  Davide Libenzi <davidel@xmailserver.org>
+ *
+ */
 
 #if defined(WIN32)
 
@@ -48,7 +67,7 @@ char const     *SysErrStr(void)
 
 
 #else           // #if defined(WIN32)
-#if defined(__LINUX__) || defined(__SOLARIS__)
+#if defined(__LINUX__) || defined(__SOLARIS__) || defined(__FREEBSD__)
 
 #include <sys/stat.h>
 #include <stdio.h>
@@ -110,6 +129,8 @@ char const     *SysErrStr(void)
 
 
 #define MAX_MB_SIZE             10000
+#define StrNCpy(t, s, n)        do { strncpy(t, s, n); (t)[(n) - 1] = '\0'; } while (0)
+#define StrSNCpy(t, s)          StrNCpy(t, s, sizeof(t))
 
 
 
@@ -312,8 +333,11 @@ int             main(int argc, char *argv[])
                     szAutoUsr[128] = "mkuser",
                     szUsrLine[1024] = "";
 
-    for (ii = 1; (ii < argc) && (argv[ii][0] == '-'); ii++)
+    for (ii = 1; ii < argc; ii++)
     {
+        if (argv[ii][0] != '-')
+            continue;
+
         switch (argv[ii][1])
         {
             case ('a'):
@@ -323,22 +347,22 @@ int             main(int argc, char *argv[])
 
             case ('d'):
                 if (++ii < argc)
-                    strcpy(szAutoDomain, argv[ii]);
+                    StrSNCpy(szAutoDomain, argv[ii]);
                 break;
 
             case ('f'):
                 if (++ii < argc)
-                    strcpy(szInputFile, argv[ii]);
+                    StrSNCpy(szInputFile, argv[ii]);
                 break;
 
             case ('u'):
                 if (++ii < argc)
-                    strcpy(szAutoUsr, argv[ii]);
+                    StrSNCpy(szAutoUsr, argv[ii]);
                 break;
 
             case ('r'):
                 if (++ii < argc)
-                    strcpy(szRootDir, argv[ii]);
+                    StrSNCpy(szRootDir, argv[ii]);
                 break;
 
             case ('s'):
@@ -353,7 +377,7 @@ int             main(int argc, char *argv[])
 
             case ('m'):
                 bMaildir = true;
-                return (0);
+                break;
 
             case ('h'):
                 ShowUsage(argv[0]);
