@@ -1,6 +1,6 @@
 /*
- *  MailSvr by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999  Davide Libenzi
+ *  XMail by Davide Libenzi ( Intranet and Internet mail server )
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  Davide Libenzi <davide_libenzi@mycio.com>
+ *  Davide Libenzi <davidel@xmailserver.org>
  *
  */
 
@@ -30,6 +30,7 @@
 #include "SList.h"
 #include "BuffSock.h"
 #include "MailConfig.h"
+#include "MessQueue.h"
 #include "MailSvr.h"
 #include "MiscUtils.h"
 #include "SvrUtils.h"
@@ -42,6 +43,7 @@
 
 
 #define SVR_LINKS_FILE              "pop3links.tab"
+#define SVR_LINKS_ENABLE_DIR        "pop3links"
 #define SVR_POP3LOCKS_DIR           "pop3linklocks"
 #define LINKS_TABLE_LINE_MAX        2048
 
@@ -76,8 +78,9 @@ struct GwLkDBScanData
 
 
 
-static char    *GwLkGetTableFilePath(char *pszLnkFilePath);
-static char    *GwLkGetLocksDir(char *pszLocksDir);
+static char    *GwLkGetTableFilePath(char *pszLnkFilePath, int iMaxPath);
+static char    *GwLkEnableDir(char *pszEnableDir, int iMaxPath);
+static char    *GwLkGetLocksDir(char *pszLocksDir, int iMaxPath);
 static POP3Link *GwLkGetLinkFromStrings(char **ppszStrings);
 static int      GwLkWriteLink(FILE * pLnkFile, POP3Link * pPopLnk);
 static char    *GwLkGetLockFileName(POP3Link const * pPopLnk, char *pszLockFile);
@@ -91,12 +94,13 @@ static int      GwLkGetDisableFilePath(POP3Link const * pPopLnk, char *pszEnable
 
 
 
-static char    *GwLkGetTableFilePath(char *pszLnkFilePath)
+
+static char    *GwLkGetTableFilePath(char *pszLnkFilePath, int iMaxPath)
 {
 
-    CfgGetRootPath(pszLnkFilePath);
+    CfgGetRootPath(pszLnkFilePath, iMaxPath);
 
-    strcat(pszLnkFilePath, SVR_LINKS_FILE);
+    StrNCat(pszLnkFilePath, SVR_LINKS_FILE, iMaxPath);
 
     return (pszLnkFilePath);
 
@@ -104,12 +108,25 @@ static char    *GwLkGetTableFilePath(char *pszLnkFilePath)
 
 
 
-static char    *GwLkGetLocksDir(char *pszLocksDir)
+static char    *GwLkEnableDir(char *pszEnableDir, int iMaxPath)
 {
 
-    CfgGetRootPath(pszLocksDir);
+    CfgGetRootPath(pszEnableDir, iMaxPath);
 
-    strcat(pszLocksDir, SVR_POP3LOCKS_DIR);
+    StrNCat(pszEnableDir, SVR_LINKS_ENABLE_DIR, iMaxPath);
+
+    return (pszEnableDir);
+
+}
+
+
+
+static char    *GwLkGetLocksDir(char *pszLocksDir, int iMaxPath)
+{
+
+    CfgGetRootPath(pszLocksDir, iMaxPath);
+
+    StrNCat(pszLocksDir, SVR_POP3LOCKS_DIR, iMaxPath);
 
     return (pszLocksDir);
 
@@ -289,11 +306,12 @@ int             GwLkAddLink(POP3Link * pPopLnk)
 
     char            szLnkFilePath[SYS_MAX_PATH] = "";
 
-    GwLkGetTableFilePath(szLnkFilePath);
+    GwLkGetTableFilePath(szLnkFilePath, sizeof(szLnkFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -362,7 +380,7 @@ int             GwLkRemoveLink(POP3Link * pPopLnk)
 
     char            szLnkFilePath[SYS_MAX_PATH] = "";
 
-    GwLkGetTableFilePath(szLnkFilePath);
+    GwLkGetTableFilePath(szLnkFilePath, sizeof(szLnkFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -370,7 +388,8 @@ int             GwLkRemoveLink(POP3Link * pPopLnk)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -464,6 +483,21 @@ int             GwLkRemoveLink(POP3Link * pPopLnk)
 
     SysRemove(szTmpLnkFilePath);
 
+///////////////////////////////////////////////////////////////////////////////
+//  Remove the disable file if exist
+///////////////////////////////////////////////////////////////////////////////
+    char            szEnableFile[SYS_MAX_PATH] = "";
+
+    if (GwLkGetDisableFilePath(pPopLnk, szEnableFile) < 0)
+    {
+        ErrorPush();
+        RLckUnlockEX(hResLock);
+        return (ErrorPop());
+    }
+
+    CheckRemoveFile(szEnableFile);
+
+
     RLckUnlockEX(hResLock);
 
     return (0);
@@ -477,7 +511,7 @@ int             GwLkRemoveUserLinks(const char *pszDomain, const char *pszName)
 
     char            szLnkFilePath[SYS_MAX_PATH] = "";
 
-    GwLkGetTableFilePath(szLnkFilePath);
+    GwLkGetTableFilePath(szLnkFilePath, sizeof(szLnkFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -485,7 +519,8 @@ int             GwLkRemoveUserLinks(const char *pszDomain, const char *pszName)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -592,7 +627,7 @@ int             GwLkRemoveDomainLinks(const char *pszDomain)
 
     char            szLnkFilePath[SYS_MAX_PATH] = "";
 
-    GwLkGetTableFilePath(szLnkFilePath);
+    GwLkGetTableFilePath(szLnkFilePath, sizeof(szLnkFilePath));
 
     char            szTmpFile[SYS_MAX_PATH] = "";
 
@@ -600,7 +635,8 @@ int             GwLkRemoveDomainLinks(const char *pszDomain)
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockEX(CfgGetBasedPath(szLnkFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
     {
@@ -712,11 +748,12 @@ int             GwLkGetDBFileSnapShot(const char *pszFileName)
 
     char            szGwLkFilePath[SYS_MAX_PATH] = "";
 
-    GwLkGetTableFilePath(szGwLkFilePath);
+    GwLkGetTableFilePath(szGwLkFilePath, sizeof(szGwLkFilePath));
 
 
     char            szResLock[SYS_MAX_PATH] = "";
-    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szGwLkFilePath, szResLock));
+    RLCK_HANDLE     hResLock = RLckLockSH(CfgGetBasedPath(szGwLkFilePath, szResLock,
+                            sizeof(szResLock)));
 
     if (hResLock == INVALID_RLCK_HANDLE)
         return (ErrGetErrorCode());
@@ -851,12 +888,23 @@ POP3Link       *GwLkGetNextUser(GWLKF_HANDLE hLinksDB)
 static char    *GwLkGetLockFileName(POP3Link const * pPopLnk, char *pszLockFile)
 {
 
-    char            szLocksDir[SYS_MAX_PATH] = "";
+    char            szLocksDir[SYS_MAX_PATH] = "",
+                    szRmtDomain[MAX_HOST_NAME] = "";
 
-    GwLkGetLocksDir(szLocksDir);
+    GwLkGetLocksDir(szLocksDir, sizeof(szLocksDir));
+
+    StrSNCpy(szRmtDomain, pPopLnk->pszRmtDomain);
+
+///////////////////////////////////////////////////////////////////////////////
+//  Sanitize the ':' coming from the port specification
+///////////////////////////////////////////////////////////////////////////////
+    char           *pszColon = strchr(szRmtDomain, ':');
+
+    if (pszColon != NULL)
+        *pszColon = '_';
 
     sprintf(pszLockFile, "%s%s%s@%s", szLocksDir, SYS_SLASH_STR, pPopLnk->pszRmtName,
-            pPopLnk->pszRmtDomain);
+            szRmtDomain);
 
     return (pszLockFile);
 
@@ -895,7 +943,7 @@ int             GwLkClearLinkLocksDir(void)
 
     char            szLocksDir[SYS_MAX_PATH] = "";
 
-    GwLkGetLocksDir(szLocksDir);
+    GwLkGetLocksDir(szLocksDir, sizeof(szLocksDir));
 
     return (MscClearDirectory(szLocksDir));
 
@@ -907,7 +955,7 @@ int             GwLkLocalDomain(POP3Link const * pPopLnk)
 {
 
     return (((pPopLnk != NULL) && (pPopLnk->pszDomain[0] != '@') &&
-            (pPopLnk->pszDomain[0] != '?') && (pPopLnk->pszDomain[0] != '&')) ? 1: 0);
+                    (pPopLnk->pszDomain[0] != '?') && (pPopLnk->pszDomain[0] != '&')) ? 1 : 0);
 
 }
 
@@ -917,7 +965,7 @@ int             GwLkMasqueradeDomain(POP3Link const * pPopLnk)
 {
 
     return (((pPopLnk != NULL) &&
-            ((pPopLnk->pszDomain[0] == '?') || (pPopLnk->pszDomain[0] == '&'))) ? 1: 0);
+                    ((pPopLnk->pszDomain[0] == '?') || (pPopLnk->pszDomain[0] == '&'))) ? 1 : 0);
 
 }
 
@@ -926,24 +974,35 @@ int             GwLkMasqueradeDomain(POP3Link const * pPopLnk)
 static int      GwLkGetDisableFilePath(POP3Link const * pPopLnk, char *pszEnableFile)
 {
 
-    UserInfo       *pUI = UsrGetUserByName(pPopLnk->pszDomain, pPopLnk->pszName);
-
-    if (pUI == NULL)
-        return (ErrGetErrorCode());
-
-    char            szUserPath[SYS_MAX_PATH] = "";
-
-    if (UsrGetUserPath(pUI, szUserPath) == NULL)
+    if (GwLkLocalDomain(pPopLnk))
     {
-        ErrorPush();
+        UserInfo       *pUI = UsrGetUserByName(pPopLnk->pszDomain, pPopLnk->pszName);
+
+        if (pUI == NULL)
+            return (ErrGetErrorCode());
+
+        char            szUserPath[SYS_MAX_PATH] = "";
+
+        if (UsrGetUserPath(pUI, szUserPath, sizeof(szUserPath), 1) == NULL)
+        {
+            ErrorPush();
+            UsrFreeUserInfo(pUI);
+            return (ErrorPop());
+        }
+
         UsrFreeUserInfo(pUI);
-        return (ErrorPop());
+
+        sprintf(pszEnableFile, "%s%s@%s.disabled", szUserPath, pPopLnk->pszRmtName, pPopLnk->pszRmtDomain);
     }
+    else
+    {
+        char            szEnableDir[SYS_MAX_PATH] = "";
 
-    UsrFreeUserInfo(pUI);
+        GwLkEnableDir(szEnableDir, sizeof(szEnableDir));
 
-
-    sprintf(pszEnableFile, "%s%s@%s.disabled", szUserPath, pPopLnk->pszRmtName, pPopLnk->pszRmtDomain);
+        sprintf(pszEnableFile, "%s%s%s@%s.disabled", szEnableDir, SYS_SLASH_STR,
+                pPopLnk->pszRmtName, pPopLnk->pszRmtDomain);
+    }
 
     return (0);
 
@@ -999,10 +1058,12 @@ int             GwLkEnable(POP3Link const * pPopLnk, bool bEnable)
 
         time(&tCurr);
 
-        struct tm      *ptmTime = localtime(&tCurr);
+        struct tm       tmTime;
         char            szTime[256] = "";
 
-        strftime(szTime, sizeof(szTime), "%d %b %Y %H:%M:%S %Z", ptmTime);
+        SysLocalTime(&tCurr, &tmTime);
+
+        strftime(szTime, sizeof(szTime), "%d %b %Y %H:%M:%S %Z", &tmTime);
 
         fprintf(pFile, "%s\n", szTime);
 
@@ -1034,7 +1095,7 @@ int             GwLkEnable(char const * pszDomain, char const * pszName,
         if ((stricmp(pPopLnk->pszDomain, pszDomain) == 0) &&
                 (stricmp(pPopLnk->pszName, pszName) == 0) &&
                 ((pszRmtDomain == NULL) || (stricmp(pPopLnk->pszRmtDomain, pszRmtDomain) == 0)) &&
-                ((pszRmtName == NULL) || (stricmp(pPopLnk->pszRmtDomain, pszRmtName) == 0)))
+                ((pszRmtName == NULL) || (stricmp(pPopLnk->pszRmtName, pszRmtName) == 0)))
         {
             GwLkEnable(pPopLnk, bEnable);
 
