@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 
 
 
-char           *UAthGetRootPath(char const * pszService, char *pszAuthPath);
+char           *UAthGetRootPath(char const * pszService, char *pszAuthPath, int iMaxPath);
 int             UAthAuthenticateUser(char const * pszService, char const * pszDomain,
                         char const * pszUsername, char const * pszPassword);
 int             UAthAddUser(char const * pszService, UserInfo * pUI);
