@@ -1,6 +1,6 @@
 /*
  *  XMail by Davide Libenzi ( Intranet and Internet mail server )
- *  Copyright (C) 1999,2000,2001  Davide Libenzi
+ *  Copyright (C) 1999,...,2002  Davide Libenzi
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 
 
 
-
 #define INVALID_USRF_HANDLE         ((USRF_HANDLE) 0)
 #define INVALID_ALSF_HANDLE         ((ALSF_HANDLE) 0)
 
@@ -75,13 +74,15 @@ typedef struct ALSF_HANDLE_struct
 
 int             UsrCheckUsersIndexes(void);
 int             UsrCheckAliasesIndexes(void);
-char           *UsrGetMLTableFilePath(UserInfo * pUI, char *pszMLTablePath);
+char           *UsrGetMLTableFilePath(UserInfo * pUI, char *pszMLTablePath, int iMaxPath);
 UserType        UsrGetUserType(UserInfo * pUI);
 UserInfo       *UsrCreateDefaultUser(char const * pszDomain, char const * pszName,
                         char const * pszPassword, UserType TypeUser);
 void            UsrFreeUserInfo(UserInfo * pUI);
 char           *UsrGetUserInfoVar(UserInfo * pUI, const char *pszName,
                         const char *pszDefault = NULL);
+int             UsrGetUserInfoVarInt(UserInfo * pUI, const char *pszName, int iDefault);
+int             UsrDelUserInfoVar(UserInfo * pUI, const char *pszName);
 int             UsrSetUserInfoVar(UserInfo * pUI, const char *pszName,
                         const char *pszValue);
 char          **UsrGetProfileVars(UserInfo * pUI);
@@ -93,6 +94,7 @@ void            UsrFreeAlias(AliasInfo * pAI);
 int             UsrAddAlias(AliasInfo * pAI);
 int             UsrRemoveAlias(const char *pszDomain, const char *pszAlias);
 int             UsrRemoveDomainAliases(const char *pszDomain);
+UserInfo       *UsrLookupUser(const char *pszDomain, const char *pszName);
 UserInfo       *UsrGetUserByName(const char *pszDomain, const char *pszName);
 UserInfo       *UsrGetUserByNameOrAlias(const char *pszDomain, const char *pszName,
                         char *pszRealAddr = NULL);
@@ -110,8 +112,8 @@ UserInfo       *UsrGetNextUser(USRF_HANDLE hUsersDB);
 int             UsrPOP3Lock(UserInfo * pUI);
 void            UsrPOP3Unlock(UserInfo * pUI);
 int             UsrClearPop3LocksDir(void);
-char           *UsrGetUserPath(UserInfo * pUI, char *pszUserPath, int iFinalSlash = 1);
-char           *UsrGetMailboxPath(UserInfo * pUI, char *pszMBPath, int iFinalSlash = 1);
+char           *UsrGetUserPath(UserInfo * pUI, char *pszUserPath, int iMaxPath, int iFinalSlash);
+char           *UsrGetMailboxPath(UserInfo * pUI, char *pszMBPath, int iMaxPath, int iFinalSlash);
 int             UsrMoveToMailBox(UserInfo * pUI, char const * pszFileName,
                         char const * pszMessageID);
 int             UsrGetMailProcessFile(UserInfo * pUI, char *pszMPPath);
